/***********************************************************************************
 *  Copyright 2013 by Shantanu Tushar <shantanu@kde.org>                           *
 *                                                                                 *
 *   This library is free software; you can redistribute it and/or                 *
 *   modify it under the terms of the GNU Lesser General Public                    *
 *   License as published by the Free Software Foundation; either                  *
 *   version 2.1 of the License, or (at your option) any later version.            *
 *                                                                                 *
 *   This library is distributed in the hope that it will be useful,               *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
 *   Lesser General Public License for more details.                               *
 *                                                                                 *
 *   You should have received a copy of the GNU Lesser General Public              *
 *   License along with this library.  If not, see <http://www.gnu.org/licenses/>. *
 ***********************************************************************************/

#include "objectpair.h"

ObjectPair::ObjectPair(QObject* parent): QObject(parent)
{
}

QObject* ObjectPair::first() const
{
    return m_pair.first;
}

QObject* ObjectPair::second() const
{
    return m_pair.second;
}

void ObjectPair::setFirst(QObject* object)
{
    if (m_pair.first != object) {
        m_pair.first = object;
        emit firstChanged();
    }
}

void ObjectPair::setSecond(QObject* object)
{
    if (m_pair.second != object) {
        m_pair.second = object;
        emit secondChanged();
    }
}
