/***************************************************************************
 *   Copyright 2011 Sinny Kumari <ksinny@gmail.com>                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef METADATAPICTUREMODEL_H
#define METADATAPICTUREMODEL_H

#include "mediacenter/pmcmetadatamodel.h"

class MetadataPictureModel : public PmcMetadataModel
{
    Q_OBJECT
public:
    explicit MetadataPictureModel (QObject* parent = 0);
    virtual ~MetadataPictureModel();

    virtual QVariant data (const QModelIndex& index, int role = Qt::DisplayRole) const;
};

#endif // METADATAPICTUREMODEL_H
