/***********************************************************************************
 *   Copyright Flavio Tordini <flavio.tordini@gmail.com>                           *
 *                                                                                 *
 *   This library is free software; you can redistribute it and/or                 *
 *   modify it under the terms of the GNU Lesser General Public                    *
 *   License as published by the Free Software Foundation; either                  *
 *   version 2.1 of the License, or (at your option) any later version.            *
 *                                                                                 *
 *   This library is distributed in the hope that it will be useful,               *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
 *   Lesser General Public License for more details.                               *
 *                                                                                 *
 *   You should have received a copy of the GNU Lesser General Public              *
 *   License along with this library.  If not, see <http://www.gnu.org/licenses/>. *
 ***********************************************************************************/

#ifndef VIDEODEFINITION_H
#define VIDEODEFINITION_H

#include <QHash>
#include <QStringList>

class VideoDefinition {

public:
    static QStringList getDefinitionNames();
    static QList<int> getDefinitionCodes();
    static QHash<QString, int> getDefinitions();
    static int getDefinitionCode(QString name);
    static QString getDefinitionName(int code);

};

#endif // VIDEODEFINITION_H
