/***********************************************************************************
 *   Copyright 2014 Shantanu Tushar <shantanu@kde.org>                             *
 *                                                                                 *
 *                                                                                 *
 *   This library is free software; you can redistribute it and/or                 *
 *   modify it under the terms of the GNU Lesser General Public                    *
 *   License as published by the Free Software Foundation; either                  *
 *   version 2.1 of the License, or (at your option) any later version.            *
 *                                                                                 *
 *   This library is distributed in the hope that it will be useful,               *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of                *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
 *   Lesser General Public License for more details.                               *
 *                                                                                 *
 *   You should have received a copy of the GNU Lesser General Public              *
 *   License along with this library.  If not, see <http://www.gnu.org/licenses/>. *
 ***********************************************************************************/

#include "eventspicturesmodel.h"

EventsPicturesModel::EventsPicturesModel(QObject* parent, MediaLibrary* mediaLibrary): PmcMetadataModel(parent, mediaLibrary)
{
    showMediaType(MediaCenter::Picture);
}

QVariant EventsPicturesModel::data(const QModelIndex& index, int role) const
{
    switch (role) {
    case Qt::DecorationRole:
        return PmcMetadataModel::data (index, MediaCenter::MediaUrlRole);
    case MediaCenter::MediaTypeRole:
        return "image";
    case MediaCenter::HideLabelRole:
        return true;
    case MediaCenter::DecorationTypeRole:
        return "image";
    case MediaCenter::IsExpandableRole:
        return false;
    }

    return PmcMetadataModel::data(index, role);
}
